HeartbeatSensorBase = HeartbeatSensorBase or class(WeaponGadgetBase)
function HeartbeatSensorBase:init(unit,...)
	HeartbeatSensorBase.super.init(self,unit,...)
	self._unit = self._unit or unit
	self._on = false
	self._npc = false
	
	self._on_event = self._on_event or "gadget_flashlight_on"
	self._off_event = self._off_event or "gadget_flashlight_off"
	
	if self._BATTERY_ENABLED == nil then 
		self._BATTERY_ENABLED = false
	end
	self._BATTERY_LEVEL_START = self._BATTERY_LEVEL_START or 1 --[0-1]
	self._BATTERY_DURATION = self._BATTERY_DURATION or 60 --seconds to drain from full
	if self._BATTERY_CAN_AUTO_RECHARGE == nil then
		self._BATTERY_CAN_AUTO_RECHARGE = false
	end
	self._BATTERY_AUTO_RECHARGE_RATE = self._BATTERY_AUTO_RECHARGE_RATE or self._BATTERY_DURATION --seconds to charge from empty
	self._BATTERY_MIN_THRESHOLD = self._BATTERY_MIN_THRESHOLD or ((self._BATTERY_AUTO_RECHARGE_RATE / 5) / 100) --must be at least 1/5 charged to turn on
	self._BATTERY_REPLENISHED_BY_AMMO_PICKUP = self._BATTERY_REPLENISHED_BY_AMMO_PICKUP or 0.5 --[0-1]
	self._battery_level = self._BATTERY_LEVEL_START
	
	if self._AFFECTED_BY_ECM == nil then
		self._AFFECTED_BY_ECM = false
	end
	
	self._heartbeat_sensor_gui = self._unit:heartbeat_sensor_gui()
	self._unit:set_extension_update_enabled(Idstring("base"),true)
	if managers.player and self._BATTERY_ENABLED and self._BATTERY_REPLENISHED_BY_AMMO_PICKUP ~= 0 then 
		managers.player._message_system:register(Message.OnAmmoPickup, tostring(unit:key()) .. "_onammopickup", callback(self,self,"_replenish_battery"))
	end
	if self._heartbeat_sensor_gui then 
		self._heartbeat_sensor_gui:set_battery_visible(self:uses_battery())
	end
end

function HeartbeatSensorBase:set_npc(state,...)
	HeartbeatSensorBase.super.set_npc(self,state,...)
	self._npc = state
end

function HeartbeatSensorBase:set_state(on, sound_source, current_state)
	if not self:is_bipod() and not self:is_underbarrel() then
		self:_set_on(on, sound_source)
	end

	if current_state and self._heartbeat_sensor_gui then 
		self._heartbeat_sensor_gui:set_user_unit(current_state._unit)
	end

	self:_check_state(current_state)
end

function HeartbeatSensorBase:_set_on(on, sound_source)
	if self._on ~= on and sound_source and (self._on_event or self._off_event) then
		sound_source:post_event(on and self._on_event or self._off_event)
	end

	self._on = on
end

function HeartbeatSensorBase:update(unit,t,dt)
	local affected_by_ecm = self._AFFECTED_BY_ECM
	if affected_by_ecm then 
	--[[
		if affected_by_ecm == 1 then --loud
			affected_by_ecm = managers.groupai:state() and not managers.groupai:state():whisper_mode()
		elseif affected_by_ecm == 2 then --stealth only
			affected_by_ecm = managers.groupai:state() and managers.groupai:state():whisper_mode()
		end
		--]]
		local is_jammed = affected_by_ecm and managers.groupai and managers.groupai:state():is_ecm_jammer_active("camera") 
		if is_jammed then 
			if self._heartbeat_sensor_gui:is_on() then 
				self._heartbeat_sensor_gui:set_jammed(true)
			end
		end
	end
		
	if self:uses_battery() then 
		if self._heartbeat_sensor_gui:is_on() then 
			self:change_battery_level(-dt / self._BATTERY_DURATION)
			if self:is_battery_empty() then 
				self._heartbeat_sensor_gui:set_sensor_off()
			end
		elseif not is_jammed and self:can_battery_auto_recharge() and self._battery_level < 1 then 
			self:change_battery_level(dt / self._BATTERY_AUTO_RECHARGE_RATE)
		end

		self._heartbeat_sensor_gui:update_battery_level(self._battery_level,t,dt)
	end
end

function HeartbeatSensorBase:is_usable()
	if not self._heartbeat_sensor_gui then
		return false
	elseif self:is_battery_empty() then 
		return false
	end

	return true
end

function HeartbeatSensorBase:set_on()
	self._on = true

	self:_check_state()
end

function HeartbeatSensorBase:set_off()
	self._on = false

	self:_check_state()
end

function HeartbeatSensorBase:toggle()
	self._on = not self._on

	self:_check_state()
end

function HeartbeatSensorBase:is_on()
	return self._on
end

function HeartbeatSensorBase:toggle_requires_stance_update()
	return false
end

function HeartbeatSensorBase:_check_state(current_state,skip_reset_t,...)
	HeartbeatSensorBase.super._check_state(self,current_state)
	
	if self._heartbeat_sensor_gui then
		self._heartbeat_sensor_gui:toggle_sensor(self:is_on() and self:is_usable(),skip_reset_t,...)
		if self._heartbeat_sensor_gui._ws then 
			self._heartbeat_sensor_gui._ws:show()
		end
	end
end

function HeartbeatSensorBase:is_bipod()
	return false
end

function HeartbeatSensorBase:is_underbarrel()
	return false
end

function HeartbeatSensorBase:overrides_weapon_firing()
	return false
end

function HeartbeatSensorBase:destroy(unit,...)
	if managers.player then  
		managers.player._message_system:unregister(Message.OnEnemyShot, tostring(self._unit:key()) .. "_onammopickup")
	end
	HeartbeatSensorBase.super.pre_destroy(self, unit,...)
end

function HeartbeatSensorBase:is_battery_full()
	return self:get_battery_level() >= 1
end

function HeartbeatSensorBase:is_battery_empty()
	if not self:uses_battery() then 
		return false
	end
	return self:get_battery_level() <= self._BATTERY_MIN_THRESHOLD
end

function HeartbeatSensorBase:uses_battery()
	return self._BATTERY_ENABLED
end

function HeartbeatSensorBase:get_battery_level()
	return self._battery_level
end

function HeartbeatSensorBase:can_battery_auto_recharge()
	return self._BATTERY_CAN_AUTO_RECHARGE
end

function HeartbeatSensorBase:_replenish_battery()
	self:change_battery_level(self._BATTERY_REPLENISHED_BY_AMMO_PICKUP,true)
end

function HeartbeatSensorBase:change_battery_level(value)
	self._battery_level = math.clamp(self._battery_level + value,0,1)
	self:_check_state(nil,true)
end

function HeartbeatSensorBase:set_battery_level(value)
	self._battery_level = math.clamp(value,0,1)
	self:_check_state()
end
